indexing

	description:

		"Objects that represent a deferred error, to be raised only if executed"

	library: "Gobo Eiffel XSLT Library"
	copyright: "Copyright (c) 2004, Colin Adams and others"
	license: "MIT License"
	date: "$Date: 2007-05-06 02:55:32 -0700 (Sun, 06 May 2007) $"
	revision: "$Revision: 5959 $"

class XM_XSLT_DEFERRED_ERROR

inherit

	XM_XPATH_COMPUTED_EXPRESSION
		redefine
			evaluate_item, create_iterator, create_node_iterator, is_deferred_error
		end

create

	make

feature {NONE} -- Initialization

	make (an_error: XM_XPATH_ERROR_VALUE; an_instruction_name: STRING) is
			-- Establish invariant.
		require
			error_not_void: an_error /= Void
			instruction_name_not_void: an_instruction_name /= Void
		do
			error := an_error
			instruction_name := an_instruction_name
			compute_static_properties
			initialized := True
		ensure
			error_set: error = an_error
			name_set: instruction_name = an_instruction_name
		end

feature -- Access

	instruction_name: STRING
			-- Name of instruction, for diagnostics (??)

	item_type: XM_XPATH_ITEM_TYPE is
			-- Data type of the expression, when known;
		do
			Result := any_item
			if Result /= Void then
				-- Bug in SE 1.0 and 1.1: Make sure that
				-- that `Result' is not optimized away.
			end
		end

feature -- Status report

	display (a_level: INTEGER) is
			-- Diagnostic print of expression structure to `std.error'
		do
			todo ("display", False)
		end

	is_deferred_error: BOOLEAN is
			-- Is `Current' a deferred error?
		do
			Result := True
		end

feature -- Optimization

	check_static_type (a_context: XM_XPATH_STATIC_CONTEXT; a_context_item_type: XM_XPATH_ITEM_TYPE) is
			-- Perform static type-checking of `Current' and its subexpressions.
		do
			mark_unreplaced
		end

	optimize (a_context: XM_XPATH_STATIC_CONTEXT; a_context_item_type: XM_XPATH_ITEM_TYPE) is
			-- Perform optimization of `Current' and its subexpressions.
		do
			mark_unreplaced
		end

feature -- Evaluation

	evaluate_item (a_result: DS_CELL [XM_XPATH_ITEM]; a_context: XM_XPATH_CONTEXT) is
			-- Evaluate as a single item to `a_result'.
		do
			a_result.put (create {XM_XPATH_INVALID_ITEM}.make (error_value))
		end

	create_iterator (a_context: XM_XPATH_CONTEXT) is
			-- Iterator over the values of a sequence
		do
			create {XM_XPATH_INVALID_ITERATOR} last_iterator.make (error)
		end

	create_node_iterator (a_context: XM_XPATH_CONTEXT) is
			-- Create iterator over a sequence of nodes.
		do
			create {XM_XPATH_INVALID_NODE_ITERATOR} last_node_iterator.make (error)
		end

	
feature {XM_XPATH_EXPRESSION} -- Restricted

	compute_cardinality is
			-- Compute cardinality.
		do
			set_cardinality_zero_or_more
		end

feature {NONE} -- Implementation

	error: XM_XPATH_ERROR_VALUE
			-- Error to report

invariant

	error_not_void: initialized implies error /= Void

end
	
