indexing

	description:

		"Objects that compare text strings"

	library: "Gobo Eiffel XSLT Library"
	copyright: "Copyright (c) 2004, Colin Adams and others"
	license: "MIT License"
	date: "$Date: 2007-01-26 10:55:25 -0800 (Fri, 26 Jan 2007) $"
	revision: "$Revision: 5877 $"

class XM_XSLT_TEXT_COMPARER

inherit
	
	KL_COMPARATOR [XM_XPATH_ITEM]

	XM_XPATH_DEBUGGING_ROUTINES
		export {NONE} all end

create

	make, make_from_collator

feature {NONE} -- Initialiaztion

	make (a_comparer: KL_COMPARATOR [XM_XPATH_ITEM]) is
			-- Establish invariant.
		require
			comparer_not_void: a_comparer /= Void
		do
			comparer := a_comparer
		ensure
			comparer_set: comparer = a_comparer
		end
	
	make_from_collator (a_collator: ST_COLLATOR) is
			-- Establish invariant
		require
			collator_not_void: a_collator /= Void
		do
			collator := a_collator
		ensure
			collator_set: collator = a_collator
		end
	
feature -- Comparison

	less_than (u, v: XM_XPATH_ITEM): BOOLEAN is
			-- Is `u' considered less than `v'?
		local
			s1, s2: STRING
		do
			if collator /= Void then
				s1 := u.string_value
				s2 := v.string_value
				Result := collator.less_than (s1, s2)
			else
				Result := comparer.less_than (u, v)
			end
		end

feature {NONE} -- Implementation

	collator: ST_COLLATOR
			-- Collator used to perform string comparisons

	comparer: KL_COMPARATOR [XM_XPATH_ITEM]
			-- Comparator used to perform string comparisons

invariant

	collator_or_comparer: collator /= Void xor comparer /= Void

end
